package com.perforce.hws.p4base;

import com.perforce.p4java.exception.AccessException;
import com.perforce.p4java.exception.ConnectionException;
import com.perforce.p4java.server.IServer;

/**
 * This is how most clients should obtain CloseableServer instances.
 */
public interface CloseableServerProvider {

    /**
     * Issue the user an open server connection.
     * <p>
     * Note that important connection properties, such as charset, should be
     * created automatically as needed.
     *
     * @param id     A hierarchical-style ID that can map to more than one configuration
     *               group in our configuration settings.
     *               This can be null, in which case we look for a "default"
     *               configuration.
     * @param user   The p4d account on the server.
     *               Can be null, in case you are actually doing authentication
     *               yourself.
     * @param ticket The ticket that should have been previously issued.
     *               Can be null, for cases where your code is handling the auth.
     *
     * @return
     */
    CloseableServer obtain(String id, String user, String ticket);

    /**
     * Actually implements the "close" operation for CloseableServers.
     */
    default void release(CloseableServer closeableServer) {
        // The default implementation disconnects the server, assuming it
        // won't be reused.
        IServer server = closeableServer.get();
        if (server != null && server.isConnected()) {
            try {
                server.disconnect();
            } catch (ConnectionException | AccessException e) {
                throw new P4JavaRuntimeException(e);
            }
        }
    }
}
