package com.perforce.hws.p4base;

import com.perforce.p4java.server.IServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.*;
import java.util.function.Supplier;

/**
 * For file browsing applications, we frequently need to combine the "depots",
 * "files", "dirs", and "print" commands.
 */
public interface ListingMethods extends P4Methods, PrintMethods {

    Logger logger = LoggerFactory.getLogger(ListingMethods.class);

    default List<ResultMap> listPath(Supplier<IServer> serverSupplier,
                                     String path) throws IOException {
        if (path == null) {
            return exec(serverSupplier, "depots");
        }

        String spec = path;
        if (!spec.startsWith("//")) {
            spec = "//" + spec;
        }

        List<ResultMap> filesResults = null;

        try {
            filesResults = exec(serverSupplier, "files", spec);
        } catch (P4BaseException ex) {
            if (ex.getResultMap().getErrors().stream().anyMatch(e -> e.getUniqueCode().equals("838998116"))) {
                logger.debug("Ignoring exception", ex);
                filesResults = null;
            } else {
                throw ex;
            }
        }

        if (filesResults != null &&
                filesResults.size() == 1 &&
                filesResults.get(0).containsKey("depotFile") &&
                filesResults.get(0).get("depotFile").equals(spec)) {
            // Switch to using fstat and the file content into the result map.

            List<ResultMap> fstatResults = exec(serverSupplier, "fstat", "-Ol", spec);

            ResultMap resultMap = fstatResults.get(0);

            byte content[] = printFile(serverSupplier, spec);

            resultMap.put("content", new String(content));

            return Arrays.asList(resultMap);
        } else {
            // This is a "directory" list.
            String dirSpec = spec + "/*";

            filesResults = exec(serverSupplier, "files", "-e", dirSpec);

            List<ResultMap> dirResults = exec(serverSupplier, "dirs", dirSpec);

            List<ResultMap> results = new ArrayList<>();

            if (!filesResults.stream().anyMatch(r -> !r.getErrors().isEmpty())) {
                results.addAll(filesResults);
            }

            if (!dirResults.stream().anyMatch(r -> !r.getErrors().isEmpty())) {
                results.addAll(dirResults);
            }

            return results;
        }
    }
}
