package com.perforce.hws.p4base;

import com.perforce.hws.util.StreamUtils;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.AccessException;
import com.perforce.p4java.exception.ConnectionException;
import com.perforce.p4java.exception.RequestException;
import com.perforce.p4java.impl.generic.core.file.FileSpec;
import com.perforce.p4java.server.IServer;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;
import java.util.function.Supplier;

/**
 * Helpers to interact with "p4 print".
 */
public interface PrintMethods extends P4Methods, StreamUtils {

    default byte[] printFile(Supplier<IServer> serverSupplier, String file) throws IOException {
        // For whatever reason the execMapCmd method we call doesn't allow
        // for the print command.
        IFileSpec fileSpec = new FileSpec(file);
        List<IFileSpec> fileSpecs = Arrays.asList(fileSpec);
        try (InputStream is = serverSupplier.get().getFileContents(fileSpecs, false, true)) {
            return readAll(is);
        } catch (ConnectionException | RequestException | AccessException e) {
            throw new P4JavaRuntimeException(e);
        }
    }
}
