package com.perforce.hws.p4base;

import com.perforce.p4java.server.IServer;

import java.util.function.Supplier;

/**
 * Provides a consistent way to obtain an IServer instance.
 * <p>
 * ServerHandles should be used in lieu of IServer, since you may not know
 * exactly when you should connect to a running p4d.
 * <p>
 * The primary example is using a server handle in different layers of a
 * web application route, where you might authenticate in a filter and then
 * just issue commands during method handling. The main bit should just assume
 * that the handle exists and is connected, the filter shouldn't how the
 * connection is used later during the method. In some cases, you might decide
 * to later use a different filter, so the main handler should learn how to
 * authenticate in a different way, but not require any code changes.
 * <p>
 * We do this ourselves in deciding between using p4d for authentication or
 * Helix Cloud.
 */
public interface ServerHandle extends AutoCloseable, Supplier<IServer> {

    /**
     * We assume that the implementation will handle problems with closing server handles, or,
     * trigger a runtime exception.
     */
    void close();
}
